@@ -67,6 +67,8 @@ void catch_xcpu( int sig_num );
 void print_help( char* prog_path );
 void print_version( void );
 
+static ae_exp_manager* exp_manager = NULL;
+
 
 
 
@@ -218,23 +220,16 @@ int main( int argc, char* argv[] )
   //                          Load the simulation
   // =================================================================
   #ifndef __NO_X
-    ae_exp_manager* exp_manager = new ae_exp_manager_X11();
+    exp_manager = new ae_exp_manager_X11();
   #else
-    ae_exp_manager* exp_manager = new ae_exp_manager();
+    exp_manager = new ae_exp_manager();
   #endif
   
   exp_manager->load( num_gener, false, verbose );
   exp_manager->set_nb_gener( nb_gener );
   
-  /*ae_list_node<ae_individual*>*   indiv_node_tmp = exp_manager->get_pop()->get_indivs()->get_first();
-  ae_individual*  indiv_tmp;
-  /*for ( int32_t i = 0 ; i < exp_manager->get_pop()->get_nb_indivs() ; i++ )
-  {
-    indiv_tmp = indiv_node_tmp->get_obj();
-    printf("%le %"PRId32" %"PRId32"\n", indiv_tmp->get_fitness(), indiv_tmp->get_nb_bases_in_0_coding_RNA(), indiv_tmp->get_int_probes()[0]);
-    indiv_node_tmp = indiv_node_tmp->get_next();
-  }*/
-  
+ 
+
   // Make a numbered copy of each static input file (dynamic files are saved elsewhere)
   // TODO (?)
 
@@ -263,7 +258,10 @@ void catch_usr1( int sig_num )
   signal( SIGUSR1, catch_usr1 );
   
   printf( "display on/off\n" );
-  //~ exp_manager->toggle_display_on_off();
+  if ( exp_manager != NULL )
+  {
+    ((ae_exp_manager_X11*) exp_manager)->toggle_display_on_off();
+  }
 }
 #endif
 
@@ -295,7 +293,7 @@ void print_help( char* prog_path )
 {
   // Get the program file-name in prog_name (strip prog_path of the path)
   char* prog_name; // No new, it will point to somewhere inside prog_path
-  if ( prog_name = strrchr( prog_path, '/' ) ) prog_name++;
+  if ( ( prog_name = strrchr( prog_path, '/' )) ) prog_name++;
   else prog_name = prog_path;
   
 	printf( "******************************************************************************\n" );
@@ -313,7 +311,7 @@ void print_help( char* prog_path )
   printf( "\n" );
 	printf( "Usage : %s -h or --help\n", prog_name );
 	printf( "   or : %s -V or --version\n", prog_name );
-	printf( "   or : aevol_run_X11 [-r GENER] [-n NB_GENER] [-tvwx]\n", prog_name );
+	printf( "   or : %s [-r GENER] [-n NB_GENER] [-tvwx]\n", prog_name );
 	printf( "\nOptions\n" );
 	printf( "  -h, --help\n\tprint this help, then exit\n\n" );
 	printf( "  -V, --version\n\tprint version number, then exit\n\n" );
