@@ -163,6 +163,12 @@ class ae_exp_manager : public ae_object
                                         int16_t grid_height,
                                         ae_jumping_mt* prng );
 
+    inline void set_with_HT( bool with_HT ) ;
+    inline void set_repl_HT_with_close_points ( bool repl_HT_with_close_points) ;
+    inline void set_HT_ins_rate( double HT_ins_rate) ;
+    inline void set_HT_repl_rate( double HT_repl_rate) ;
+    inline void set_repl_HT_detach_rate( double repl_HT_detach_rate) ;
+
     // =======================================================================
     //                                 Operators
     // =======================================================================
@@ -173,7 +179,8 @@ class ae_exp_manager : public ae_object
     void write_setup_files( void );
     void save( void ) const;
     void save_copy( char* dir, int32_t num_gener = 0 ) const;
-    void load( int32_t first_gener, bool use_text_files, bool verbose, bool to_be_run = true);
+    inline void load( int32_t first_gener, bool use_text_files, bool verbose, bool to_be_run = true );
+    void load( const char* dir, int32_t first_gener, bool use_text_files, bool verbose, bool to_be_run = true );
     void load( int32_t first_gener,
                char* exp_setup_file_name,
                char* out_prof_file_name,
@@ -181,7 +188,7 @@ class ae_exp_manager : public ae_object
                char* pop_file_name,
                char* sel_file_name,
                char* sp_struct_file_name,
-               bool verbose , 
+               bool verbose, 
                bool to_be_run = true);
     void run_evolution( void );
     virtual void display( void ) {};
@@ -216,6 +223,17 @@ class ae_exp_manager : public ae_object
     //                              Protected Methods
     // =======================================================================
     inline void step_to_next_generation( void );
+
+    void load( gzFile& pop_file,
+               gzFile& env_file,
+               gzFile& exp_s_gzfile,
+               FILE*&  exp_s_txtfile,
+               gzFile& exp_backup_file,
+               gzFile& sp_struct_file,
+               gzFile& out_p_gzfile,
+               FILE*& out_p_txtfile,
+               bool verbose,
+               bool to_be_run = true );
   
     void create_missing_directories( const char* dir = "." ) const;
     void open_backup_files( gzFile& env_file,
@@ -528,6 +546,32 @@ inline void ae_exp_manager::set_spatial_structure( int16_t grid_width,
   _spatial_structure->set_prng( prng );
 }
 
+
+inline void ae_exp_manager::set_with_HT( bool with_HT )
+{
+  _exp_s->set_with_HT( with_HT );  
+}
+
+inline void ae_exp_manager::set_repl_HT_with_close_points ( bool repl_HT_with_close_points)
+{
+  _exp_s->set_repl_HT_with_close_points( repl_HT_with_close_points );
+}
+
+inline void ae_exp_manager::set_HT_ins_rate( double HT_ins_rate)
+{
+  _exp_s->set_HT_ins_rate( HT_ins_rate );
+}
+
+inline void ae_exp_manager::set_HT_repl_rate( double HT_repl_rate)
+{
+  _exp_s->set_HT_repl_rate( HT_repl_rate );
+}
+
+inline void ae_exp_manager::set_repl_HT_detach_rate( double repl_HT_detach_rate)
+{
+  _exp_s->set_repl_HT_detach_rate( repl_HT_detach_rate );
+}
+
 // ===========================================================================
 //                            Operators' definitions
 // ===========================================================================
@@ -548,4 +592,14 @@ inline void ae_exp_manager::step_to_next_generation( void )
 }
 
 
+/*!
+  \brief Load an experiment with default files from the current directory
+ */
+inline void ae_exp_manager::load( int32_t first_gener, bool use_text_files,
+                                  bool verbose, bool to_be_run /*  = true */ )
+{
+  load( ".", first_gener, use_text_files, verbose, to_be_run );
+}
+
+
 #endif // __AE_EXP_MANAGER_H__
