@@ -80,7 +80,6 @@ ae_exp_manager_X11::ae_exp_manager_X11( void ) : ae_exp_manager()
   
   _display_on             = false;
   _handle_display_on_off  = false;
-  _quit_signal            = false;
   
   // Initialize XLib stuff
   _display  = XOpenDisplay( NULL );
@@ -142,13 +141,19 @@ ae_exp_manager_X11::~ae_exp_manager_X11( void )
 
 bool ae_exp_manager_X11::quit_signal_received( void )
 {
-  return _quit_signal;
+  return _quit_signal_received;
 }
 
 void ae_exp_manager_X11::display( void )
 {
+  // ---------------------
+  // 1) Handle user events
+  // ---------------------
+  if ( _display_on ) handle_events();
+
+
   // --------------------------------------------------
-  // 1) Handle signal that toggle the display on or off
+  // 2) Handle signal that toggle the display on or off
   // --------------------------------------------------
   if ( _handle_display_on_off )
   {
@@ -176,10 +181,11 @@ void ae_exp_manager_X11::display( void )
           delete _win[num_win];
           _win[num_win] = NULL;
         }
-        
-        delete _win;
-        _win = NULL;
       }
+
+      XFlush( _display );
+      delete _win;
+      _win = NULL;
       
       _display_on = false;
     }
@@ -203,12 +209,6 @@ void ae_exp_manager_X11::display( void )
     }
   }
 
-
-  // ---------------------
-  // 2) Handle user events
-  // ---------------------
-  handle_events();
-
   
   // ----------
   // 3) Display
@@ -328,7 +328,7 @@ void ae_exp_manager_X11::handle_events( void )
           else if ( event.xkey.keycode == _key_codes[KEY_F3] ) num_win = 3;
           else if ( event.xkey.keycode == _key_codes[KEY_F4] ) num_win = 4;
           else if ( event.xkey.keycode == _key_codes[KEY_F5] ) num_win = 5;
-          else if ( event.xkey.keycode == _key_codes[KEY_F6] ) num_win = 6;
+          else  num_win = 6; // case where ( event.xkey.keycode == _key_codes[KEY_F6] ) 
           
           if ( get_show_window( num_win ) )
           {
@@ -375,13 +375,15 @@ void ae_exp_manager_X11::handle_events( void )
         }
         else if ( (event.xkey.state & ControlMask) && (event.xkey.keycode == _key_codes[KEY_Q]))
         {
-          _quit_signal = true;
+          printf(" Q U I T   R E Q U E S T E D\n");
+          _quit_signal_received = true;
           // We do not exit here, because it is cleaner to let the main program
           // interpret the signal and call the destructor of the simulation.
           // This ensures that the connection to the X server will be closed 
           // in a clean way, which is much better if we want to run other
           // X applications afterwards!
         }
+        
         break;
       }
       default :
