@@ -3,25 +3,25 @@
 //          Aevol - An in silico experimental evolution platform
 //
 // ****************************************************************************
-// 
+//
 // Copyright: See the AUTHORS file provided with the package or <www.aevol.fr>
 // Web: http://www.aevol.fr/
 // E-mail: See <http://www.aevol.fr/contact/>
 // Original Authors : Guillaume Beslon, Carole Knibbe, David Parsons
-// 
+//
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
 // the Free Software Foundation, either version 2 of the License, or
 // (at your option) any later version.
-// 
+//
 // This program is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
-// 
+//
 // You should have received a copy of the GNU General Public License
 // along with this program.  If not, see <http://www.gnu.org/licenses/>.
-// 
+//
 // ****************************************************************************
 
 
@@ -88,16 +88,16 @@ class ae_genetic_unit : public ae_object
     inline ae_fuzzy_set*    get_activ_contribution( void )      const;
     inline ae_fuzzy_set*    get_inhib_contribution( void )      const;
     inline ae_fuzzy_set*    get_phenotypic_contribution( void ) const;
-    
+
     inline ae_list<ae_rna*>** get_rna_list( void )                const;
     inline ae_list<ae_protein*>** get_protein_list( void ) const;
-    
-    
+
+
     // Direct DNA access
     inline const char*  get_sequence( void ) const;
     inline int32_t      get_seq_length( void ) const;
 
-    
+
     // Statistical data
     inline int32_t  get_nb_coding_RNAs( void )                    const;
     inline int32_t  get_nb_non_coding_RNAs( void )                const;
@@ -113,32 +113,40 @@ class ae_genetic_unit : public ae_object
     inline double   get_av_size_functional_genes( void )          const;
     inline double   get_overall_size_non_functional_genes( void ) const;
     inline double   get_av_size_non_functional_genes( void )      const;
-    
+
     inline int32_t  get_nb_bases_in_0_CDS( void );
     inline int32_t  get_nb_bases_in_0_functional_CDS( void );
     inline int32_t  get_nb_bases_in_0_non_functional_CDS( void );
     inline int32_t  get_nb_bases_in_0_RNA( void );
     inline int32_t  get_nb_bases_in_0_coding_RNA( void );
     inline int32_t  get_nb_bases_in_0_non_coding_RNA( void );
-    
+
     inline int32_t  get_nb_bases_non_essential( void );
     inline int32_t  get_nb_bases_non_essential_including_nf_genes( void );
-    
+
     inline int32_t  get_nb_bases_in_neutral_regions( void );
     inline int32_t  get_nb_neutral_regions( void );
     inline int32_t* get_beginning_neutral_regions( void );
     inline int32_t* get_end_neutral_regions( void );
-    
+
     inline double   get_modularity( void )                        const;
 
     inline double   get_dist_to_target_by_feature( ae_env_axis_feature feature )  const;
     inline double   get_fitness( void )                                           const;
     inline double   get_fitness_by_feature( ae_env_axis_feature feature )         const;
-    
-    
+
+    inline int32_t  get_min_gu_length( void ) const;
+    inline int32_t  get_max_gu_length( void ) const;
+
+
     inline void  set_rna_list( ae_list<ae_rna*>** new_list );
-    
-    
+
+    inline void set_min_gu_length( int32_t min_gu_length );
+    inline void set_max_gu_length( int32_t max_gu_length );
+
+    inline void set_exp_m( ae_exp_manager* exp_m );
+
+
     // =================================================================
     //                            Public Methods
     // =================================================================
@@ -146,14 +154,15 @@ class ae_genetic_unit : public ae_object
     void do_transcription( void );
     void do_translation( void );
     void compute_phenotypic_contribution( void );
-    
-    // DM: these two are identical to functions from ae_individual 
+
+    // DM: these two are identical to functions from ae_individual
     void compute_distance_to_target( ae_environment* envir );
     void compute_fitness( ae_environment* envir );
-    
+
     void reset_expression( void ); // useful for post-treatment programs
-    
+
     inline void print_rnas( void ) const;
+    void print_coding_rnas( void );
     inline static void print_rnas( ae_list<ae_rna*>** rnas );
     static void print_rnas( ae_list<ae_rna*>* rnas, ae_strand strand );
     void print_proteins( void ) const;
@@ -164,10 +173,10 @@ class ae_genetic_unit : public ae_object
     inline bool is_start( ae_strand strand, int32_t pos ) const;
     inline bool is_stop( ae_strand strand, int32_t pos ) const;
     int8_t      get_codon( ae_strand strand, int32_t pos ) const;
-    
+
     void compute_non_coding( void );
-  
-    
+
+
     void duplicate_promoters_included_in( int32_t pos_1, int32_t pos_2, ae_list<ae_rna*>** duplicated_promoters );
 
     void get_promoters_included_in( int32_t pos_1, int32_t pos_2, ae_list<ae_rna*>** promoters );
@@ -177,11 +186,11 @@ class ae_genetic_unit : public ae_object
     void get_leading_promoters_starting_before( int32_t pos, ae_list<ae_rna*>* leading_promoters );
     void get_lagging_promoters_starting_before( int32_t pos, ae_list<ae_rna*>* lagging_promoters );
     void get_lagging_promoters_starting_after( int32_t pos, ae_list<ae_rna*>* lagging_promoters );
-    
+
     void invert_promoters_included_in( int32_t pos_1, int32_t pos_2 );
     static void invert_promoters( ae_list<ae_rna*>** promoter_lists, int32_t seq_length );
     static void invert_promoters( ae_list<ae_rna*>** promoter_lists, int32_t pos_1, int32_t pos_2 ); // See WARNING
-    
+
     inline void extract_promoters_included_in( int32_t pos_1, int32_t pos_2, ae_list<ae_rna*>** extracted_promoters );
     inline void extract_promoters_starting_between( int32_t pos_1, int32_t pos_2, ae_list<ae_rna*>** extracted_promoters );
     void extract_leading_promoters_starting_between( int32_t pos_1, int32_t pos_2, ae_list<ae_rna*>* extracted_promoters );
@@ -190,27 +199,27 @@ class ae_genetic_unit : public ae_object
     void extract_leading_promoters_starting_before( int32_t pos, ae_list<ae_rna*>* extracted_promoters );
     void extract_lagging_promoters_starting_before( int32_t pos, ae_list<ae_rna*>* extracted_promoters );
     void extract_lagging_promoters_starting_after( int32_t pos, ae_list<ae_rna*>* extracted_promoters );
-    
+
     static void shift_promoters( ae_list<ae_rna*>** promoters_to_shift, int32_t delta_pos, int32_t seq_length );
     void insert_promoters( ae_list<ae_rna*>** promoters_to_insert );
     void insert_promoters_at( ae_list<ae_rna*>** promoters_to_insert, int32_t pos );
-    
+
     inline void remove_promoters_around( int32_t pos );
     inline void remove_promoters_around( int32_t pos_1, int32_t pos_2 );
     inline void remove_all_promoters( void );
-    
+
     inline void look_for_new_promoters_around( int32_t pos );
     inline void look_for_new_promoters_around( int32_t pos_1, int32_t pos_2 );
-    
+
     inline void move_all_promoters_after( int32_t pos, int32_t delta_pos );
     void move_all_leading_promoters_after( int32_t pos, int32_t delta_pos );
     void move_all_lagging_promoters_after( int32_t pos, int32_t delta_pos );
-    
+
     // Not implemented!!!
     //~ inline void duplicate_promoters_starting_between( int32_t pos_1, int32_t pos_2, int32_t delta_pos );
     //~ void duplicate_leading_promoters_starting_between( int32_t pos_1, int32_t pos_2, int32_t delta_pos );
     //~ void duplicate_lagging_promoters_starting_between( int32_t pos_1, int32_t pos_2, int32_t delta_pos );
-    
+
     inline void copy_promoters_included_in( int32_t pos_1, int32_t pos_2, ae_list<ae_rna*>** new_promoter_lists );
     inline void copy_promoters_starting_between( int32_t pos_1, int32_t pos_2, ae_list<ae_rna*>** new_promoter_lists );
     void copy_leading_promoters_starting_between( int32_t pos_1, int32_t pos_2, ae_list<ae_rna*>* new_promoter_list );
@@ -218,21 +227,25 @@ class ae_genetic_unit : public ae_object
     //~ inline void copy_all_promoters( ae_list** new_promoter_lists );
 
     void save( gzFile backup_file );
-    
+
     int32_t get_nb_terminators( void );
-    
-    //~ // set the genetic unit of all promoters to the appropriate 
+
+    //~ // set the genetic unit of all promoters to the appropriate
     //~ void reasign_promoter_genetic_unit (void);
-    
+
     #ifdef DEBUG
       void assert_promoters( void );
       void assert_promoters_order( void );
     #endif
-    
+
     bool* is_belonging_to_coding_RNA(void);
     void remove_non_coding_bases( void);
     void double_non_coding_bases(void);
 
+    // WARNING: The method below works properly only in the case of a single genetic unit (no plasmid).
+    // Translocations between different genetic units is not managed.
+    void compute_nb_of_affected_genes(ae_mutation * mut, int & nb_genes_at_breakpoints, int & nb_genes_in_segment, int & nb_genes_in_replaced_segment);
+
     // =================================================================
     //                           Public Attributes
     // =================================================================
@@ -262,14 +275,14 @@ class ae_genetic_unit : public ae_object
     //                           Protected Methods
     // =================================================================
     void init_statistical_data( void );
-    
+
     void remove_leading_promoters_starting_between( int32_t pos_1, int32_t pos_2 );
     void remove_lagging_promoters_starting_between( int32_t pos_1, int32_t pos_2 );
     void remove_leading_promoters_starting_before( int32_t pos );
     void remove_lagging_promoters_starting_before( int32_t pos );
     void remove_leading_promoters_starting_after( int32_t pos );
     void remove_lagging_promoters_starting_after( int32_t pos );
-    
+
     void look_for_new_leading_promoters_starting_between( int32_t pos_1, int32_t pos_2 );
     void look_for_new_lagging_promoters_starting_between( int32_t pos_1, int32_t pos_2 );
     void look_for_new_leading_promoters_starting_after( int32_t pos );
@@ -282,14 +295,14 @@ class ae_genetic_unit : public ae_object
     //                          Protected Attributes
     // =================================================================
     ae_exp_manager* _exp_m;
-    
+
     ae_individual*  _indiv;
     ae_dna*         _dna;
     ae_fuzzy_set*   _activ_contribution;
     ae_fuzzy_set*   _inhib_contribution;
     ae_fuzzy_set*   _phenotypic_contribution;
     // NB : _phenotypic_contribution is only an indicative value, not used for the whole phenotype computation
-    
+
     ae_list<ae_rna*>**     _rna_list;
     ae_list<ae_protein*>** _protein_list;
 
@@ -297,8 +310,8 @@ class ae_genetic_unit : public ae_object
     double* _dist_to_target_per_segment;
     double* _dist_to_target_by_feature;
     double* _fitness_by_feature;
-    double  _fitness;    
-    
+    double  _fitness;
+
     // Statistical data (intrinsic for this genetic unit)
     int32_t _nb_coding_RNAs;                // Number of coding RNAs (at least one gene on RNA)
     int32_t _nb_non_coding_RNAs;            // Number of non-coding-RNAs
@@ -310,17 +323,17 @@ class ae_genetic_unit : public ae_object
     int32_t _nb_non_fun_genes;              // Number of non-functional genes
     double  _overall_size_fun_genes;        // Average size of functional genes
     double  _overall_size_non_fun_genes;    // Average size of non-functional genes
-    
-    
+
+
     int32_t _nb_bases_in_0_CDS;                // Number of bases that are not included in any gene
     int32_t _nb_bases_in_0_functional_CDS;     // Number of bases that are not included in any non-degenerated gene
     int32_t _nb_bases_in_0_non_functional_CDS; // Number of bases that are not included in any degenerated gene
-    
+
     int32_t _nb_bases_in_0_RNA;                // Number of bases that are not included in any RNA
     int32_t _nb_bases_in_0_coding_RNA;         // Number of bases that are not included in any coding RNA
                                                // (RNAs containing at least one CDS)
     int32_t _nb_bases_in_0_non_coding_RNA;     // Number of bases that are not included in any non coding RNA
-    
+
     int32_t _nb_bases_non_essential;                    // Number of bases that are not included in any non-degenerated gene or involved in its expression
     int32_t _nb_bases_non_essential_including_nf_genes; // Number of bases that are not included in any gene or involved in its expression
     int32_t _nb_bases_in_neutral_regions;   // Number of bases that are not included in a neutral region
@@ -331,26 +344,28 @@ class ae_genetic_unit : public ae_object
     int32_t* _end_neutral_regions;          // Corresponding ends of neutral regions
 
     double  _modularity;  // Ratio between the pairwise distance between genes whose corresponding
-                          // phenotypic triangles overlap and the average intergenic distance 
+                          // phenotypic triangles overlap and the average intergenic distance
                           // (ignoring non-functional genes)
-    
-    
+
+    int32_t _min_gu_length;
+    int32_t _max_gu_length;
+
     // TODO : check and comment what it is for
     //~ int32_t _nb_promoters[2];
     //~ int32_t _nb_genes[2];
-    //~ int32_t _nb_functional_genes[2]; 
+    //~ int32_t _nb_functional_genes[2];
     //~ double  _average_gene_size;
     //~ double  _average_functional_gene_size;
     //~ int32_t _nb_coding_bp;
     //~ double  _clustering;
-    
+
     // We keep trace of what we have already computed to avoid double computation (mainly in post-treaments)
     bool _transcribed;
     bool _translated;
     bool _phenotypic_contributions_computed;
     bool _non_coding_computed;
     bool _distance_to_target_computed;
-    bool _fitness_computed; 
+    bool _fitness_computed;
 };
 
 
@@ -377,11 +392,6 @@ inline ae_list<ae_rna*>** ae_genetic_unit::get_rna_list( void ) const
   return _rna_list;
 }
 
-inline void ae_genetic_unit::set_rna_list( ae_list<ae_rna*>** new_list )
-{
-  _rna_list = new_list;
-} // TODO : erase that! // NOTE : Why?
-
 inline ae_list<ae_protein*>** ae_genetic_unit::get_protein_list( void ) const
 {
   assert( _protein_list );
@@ -589,24 +599,53 @@ inline double ae_genetic_unit::get_modularity( void ) const
 inline double ae_genetic_unit::get_dist_to_target_by_feature( ae_env_axis_feature feature ) const
 {
    assert( _distance_to_target_computed );
-  
+
   return _dist_to_target_by_feature[feature];
 }
 
 inline double ae_genetic_unit::get_fitness( void ) const
 {
   assert( _fitness_computed );
-  
+
   return _fitness;
 }
 
 inline double ae_genetic_unit::get_fitness_by_feature( ae_env_axis_feature feature ) const
 {
   assert( _fitness_computed );
-  
+
   return _fitness_by_feature[feature];
 }
 
+inline int32_t ae_genetic_unit::get_min_gu_length( void ) const
+{
+  return _min_gu_length;
+}
+
+inline int32_t ae_genetic_unit::get_max_gu_length( void ) const
+{
+  return _max_gu_length;
+}
+
+inline void ae_genetic_unit::set_rna_list( ae_list<ae_rna*>** new_list )
+{
+  _rna_list = new_list;
+} // TODO : erase that! // NOTE : Why?
+
+inline void ae_genetic_unit::set_min_gu_length( int32_t min_gu_length )
+{
+  _min_gu_length = min_gu_length;
+}
+
+inline void ae_genetic_unit::set_max_gu_length( int32_t max_gu_length )
+{
+  _max_gu_length = max_gu_length;
+}
+
+inline void ae_genetic_unit::set_exp_m( ae_exp_manager* exp_m )
+{
+  _exp_m = exp_m;
+}
 
 // =====================================================================
 //                       Inline functions' definition
@@ -666,7 +705,7 @@ inline void ae_genetic_unit::extract_promoters_starting_between( int32_t pos_1,
 
   Remove promoters that include BOTH the base before AND after pos (marked X in the cartoon below).
   If the genome is smaller than the size of a promoter, all the promoters will be removed.
-  
+
   \verbatim
      -------------------------------------------------------
     |   |   |   |   | X | X |   |   |   |   |   |   |   |   |
@@ -724,7 +763,7 @@ inline void ae_genetic_unit::remove_promoters_around( int32_t pos_1, int32_t pos
 
   Look for promoters that include BOTH the base before AND after pos (marked X in the cartoon below).
   If the genome is smaller than the size of a promoter, no search is performed.
-  
+
   \verbatim
      -------------------------------------------------------
     |   |   |   |   | X | X |   |   |   |   |   |   |   |   |
@@ -736,7 +775,7 @@ inline void ae_genetic_unit::remove_promoters_around( int32_t pos_1, int32_t pos
 inline void ae_genetic_unit::look_for_new_promoters_around( int32_t pos )
 {
   assert( pos >= 0 && pos <= _dna->get_length() );
-  
+
   if ( _dna->get_length() >= PROM_SIZE )
   {
     look_for_new_leading_promoters_starting_between(  ae_utils::mod(pos - PROM_SIZE + 1, _dna->get_length()),
@@ -754,7 +793,7 @@ inline void ae_genetic_unit::look_for_new_promoters_around( int32_t pos )
                             * include BOTH the base before AND after pos_2 (marked Y in the cartoon below).
                             * are completely contained between pos_1 and pos_2.
   If the genome is smaller than the size of a promoter, no search is performed.
-  
+
   \verbatim
      -------------------------------------------------------
     |   |   |   |   | X | X |   |   |   | Y | Y |   |   |   |
@@ -801,15 +840,15 @@ inline void ae_genetic_unit::copy_promoters_starting_between( int32_t pos_1, int
 //~ inline void ae_genetic_unit::copy_all_promoters( ae_list<ae_rna*>** new_promoter_lists )
 //~ {
   //~ ae_list_node<ae_rna*>* rna_node = NULL;
-  
+
   //~ for ( int8_t strand = LEADING ; strand <= LAGGING ; strand++ )
   //~ {
     //~ rna_node = _rna_list[strand]->get_first();
-    
+
     //~ while ( rna_node != NULL )
     //~ {
       //~ new_promoter_lists[strand]->add( new ae_rna( this, *(rna_node->get_obj()) ) );
-      
+
       //~ rna_node = rna_node->get_next();
     //~ }
   //~ }
