@@ -3,25 +3,25 @@
 //          Aevol - An in silico experimental evolution platform
 //
 // ****************************************************************************
-// 
+//
 // Copyright: See the AUTHORS file provided with the package or <www.aevol.fr>
 // Web: http://www.aevol.fr/
 // E-mail: See <http://www.aevol.fr/contact/>
 // Original Authors : Guillaume Beslon, Carole Knibbe, David Parsons
-// 
+//
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
 // the Free Software Foundation, either version 2 of the License, or
 // (at your option) any later version.
-// 
+//
 // This program is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
-// 
+//
 // You should have received a copy of the GNU General Public License
 // along with this program.  If not, see <http://www.gnu.org/licenses/>.
-// 
+//
 // ****************************************************************************
 
 
@@ -70,7 +70,7 @@ class ae_grid_cell;
 class ae_individual : public ae_object
 {
   friend class ae_dna;
-  
+
   public :
 
     // =================================================================
@@ -84,12 +84,12 @@ class ae_individual : public ae_object
                     int32_t min_genome_length,
                     int32_t max_genome_length,
                     bool allow_plasmids,
-                    int32_t plasmid_minimal_length,
                     int32_t id,
+                    char* strain_name,
                     int32_t age );
-  
+
     ae_individual( ae_exp_manager* exp_m, gzFile backup_file );
-    ae_individual( const ae_individual &model );
+    ae_individual( const ae_individual &model, bool replication_report_copy = false );
     ae_individual(  ae_individual* const parent, int32_t id,
                     ae_jumping_mt* mut_prng, ae_jumping_mt* stoch_prng );
 
@@ -101,6 +101,9 @@ class ae_individual : public ae_object
     // =================================================================
     //                        Accessors: Getters
     // =================================================================
+    inline const char*  get_strain_name(void) const;
+    inline int32_t      get_age(void) const;
+
     inline ae_exp_manager*  get_exp_m( void ) const;
     inline int16_t          get_nb_genetic_units( void ) const;
     inline int32_t          get_amount_of_dna( void ) const;
@@ -112,45 +115,44 @@ class ae_individual : public ae_object
     inline bool             get_placed_in_population( void ) const;
     inline ae_jumping_mt*   get_mut_prng( void ) const;
     inline ae_jumping_mt*   get_stoch_prng( void ) const;
-    
+
     inline ae_list<ae_genetic_unit*>* get_genetic_unit_list( void )     const;
     inline const char* get_genetic_unit_sequence   ( int16_t num_unit ) const;
     inline int32_t     get_genetic_unit_seq_length ( int16_t num_unit ) const;
-    
+
     inline int32_t get_id( void ) const;
     inline int32_t get_rank( void ) const;
-    
-    
+
+
     inline ae_fuzzy_set* get_phenotype_activ( void ) const;
     inline ae_fuzzy_set* get_phenotype_inhib( void ) const;
     inline ae_phenotype* get_phenotype( void )       const;
-    
+
     inline double* get_dist_to_target_by_segment( void ) const;
-    
+
     inline ae_replication_report* get_replic_report( void ) const;
-    
+
     inline ae_list<ae_protein*>*  get_protein_list( void )  const;
     inline ae_list<ae_rna*>*      get_rna_list( void )      const;
-    
+
     inline int32_t get_nb_plasmids( void )  const;
     inline int32_t get_nb_gen_units( void ) const;
-    
+
     inline double get_w_max( void ) const;
-      
+
     // Genome size constraints
     inline int32_t get_min_genome_length( void ) const;
     inline int32_t get_max_genome_length( void ) const;
-    
+
     // Plasmids settings
     inline bool     get_allow_plasmids( void ) const;
-    inline int32_t  get_plasmid_minimal_length( void ) const;
-    
+
     // Mutation rates etc...
     inline double   get_point_mutation_rate( void ) const;
     inline double   get_small_insertion_rate( void ) const;
     inline double   get_small_deletion_rate( void ) const;
     inline int16_t  get_max_indel_size( void ) const;
-    
+
     inline double get_duplication_rate( void ) const;
     inline double get_deletion_rate( void ) const;
     inline double get_translocation_rate( void ) const;
@@ -160,7 +162,7 @@ class ae_individual : public ae_object
     inline double get_deletion_proportion( void ) const;
     inline double get_translocation_proportion( void ) const;
     inline double get_inversion_proportion( void ) const;
-    
+
     // Transfer
     inline bool   get_with_4pts_trans( void ) const;
     inline bool   get_with_HT( void ) const;
@@ -168,7 +170,7 @@ class ae_individual : public ae_object
     inline double get_HT_ins_rate( void ) const;
     inline double get_HT_repl_rate( void ) const;
     inline double get_repl_HT_detach_rate( void ) const;
-    
+
     // ------------------------------------------------------------ Alignements
     inline bool  get_with_alignments(void) const;
     inline ae_align_fun_shape get_align_fun_shape( void ) const;
@@ -176,7 +178,7 @@ class ae_individual : public ae_object
     inline int16_t get_align_sigm_mean( void ) const;
     inline int16_t get_align_lin_min( void ) const;
     inline int16_t get_align_lin_max( void ) const;
-    
+
     // Maximum shift of one seq on the other
     inline int16_t get_align_max_shift( void ) const;
     // Work zone half length
@@ -185,11 +187,11 @@ class ae_individual : public ae_object
     inline int16_t get_align_match_bonus( void ) const;
     // Corresponding residues mismatch cost
     inline int16_t get_align_mismatch_cost( void ) const;
-    
-    
+
+
     // ----------------------------------------------- Phenotypic stochasticity
     inline bool get_with_stochasticity( void ) const;
-    
+
     // Statistical data
     inline int32_t  get_total_genome_size( void ); // TODO: duplicate with get_amount_of_dna?
     inline int32_t  get_nb_coding_RNAs( void );
@@ -206,7 +208,7 @@ class ae_individual : public ae_object
     inline double   get_av_size_functional_genes( void );
     inline double   get_overall_size_non_functional_genes( void );
     inline double   get_av_size_non_functional_genes( void );
-    
+
     inline int32_t  get_nb_bases_in_0_CDS( void );
     inline int32_t  get_nb_bases_in_0_functional_CDS( void );
     inline int32_t  get_nb_bases_in_0_non_functional_CDS( void );
@@ -215,41 +217,43 @@ class ae_individual : public ae_object
     inline int32_t  get_nb_bases_in_0_non_coding_RNA( void );
     inline int32_t  get_nb_bases_in_neutral_regions( void );
     inline int32_t  get_nb_neutral_regions( void );
-    
+
     inline double get_modularity( void ); // Not const
-    
+
     inline int32_t* get_int_probes ( void ) const;
     inline double* get_double_probes ( void) const;
-    
+
 
     // =================================================================
     //                        Accessors: Setters
     // =================================================================
-    inline void set_exp_m( ae_exp_manager* exp_m );
+    inline void set_strain_name(char* name);
+
+    void set_exp_m( ae_exp_manager* exp_m );
     inline void set_id( int32_t id );
     inline void set_rank( int32_t rank );
-    
+
     void set_grid_cell( ae_grid_cell* grid_cell );
     inline void set_placed_in_population( bool placed_in_population );
-    
+
     inline void set_replication_report( ae_replication_report * rep );
-    
+
     inline void reset_dist_to_target_by_segment( double * dist_to_target_by_segment );
-    
+
     inline void set_w_max( double w_max );
-    
+
     // Genome size constraints
     inline void set_min_genome_length( int32_t min_genome_length );
     inline void set_max_genome_length( int32_t max_genome_length );
-    
+
     inline void add_GU( char * &sequence, int32_t length );
-    
+
     // Mutation rates etc...
     inline void set_point_mutation_rate( double point_mutation_rate);
     inline void set_small_insertion_rate( double small_insertion_rate);
     inline void set_small_deletion_rate( double small_deletion_rate);
     inline void set_max_indel_size( int16_t max_indel_size);
-    
+
     inline void set_duplication_rate( double duplication_rate);
     inline void set_deletion_rate( double deletion_rate);
     inline void set_translocation_rate( double translocation_rate);
@@ -259,7 +263,7 @@ class ae_individual : public ae_object
     inline void set_deletion_proportion( double deletion_proportion);
     inline void set_translocation_proportion( double translocation_proportion);
     inline void set_inversion_proportion( double inversion_proportion);
-    
+
     // Transfer
     inline void set_with_4pts_trans( bool with_4pts_trans );
     inline void set_with_alignments( bool with_alignments );
@@ -268,27 +272,27 @@ class ae_individual : public ae_object
     inline void set_HT_ins_rate( double HT_ins_rate );
     inline void set_HT_repl_rate( double HT_repl_rate );
     inline void set_repl_HT_detach_rate( double repl_HT_detach_rate );
-    
+
     // ----------------------------------------------- Phenotypic stochasticity
     inline void set_with_stochasticity( bool with_stoch );
 
     inline void set_mut_prng( ae_jumping_mt* prng );
     inline void set_stoch_prng( ae_jumping_mt* prng );
-    
+
     //------------------------------------------------ Generic probes
     inline void set_int_probes ( int32_t* int_probes);
     inline void set_double_probes ( double* double_probes);
-    
-    
+
+
     // =================================================================
     //                            Public Methods
     // =================================================================
     inline void renew_dist_to_target_by_feature( void );
     inline void renew_fitness_by_feature( void );
-    
+
     void inject_GU( ae_individual* donor );
     void inject_2GUs( ae_individual* partner );
-    
+
     virtual void evaluate( ae_environment* envir = NULL );
     virtual void reevaluate( ae_environment* envir = NULL );
     inline void do_transcription_translation_folding( void );
@@ -297,23 +301,23 @@ class ae_individual : public ae_object
     void do_folding( void );
     void compute_phenotype( void );
     void compute_distance_to_target( ae_environment* envir );
-    
+
     // Computation of a "proper" fitness value (one that increases when the individual is fitter)
     // The behaviour of this function depends on many parameters and most notably on whether it is
     // a "composite" fitness or not, and on the selection scheme.
     void compute_fitness( ae_environment* envir );
-    
+
     void compute_statistical_data();
     void compute_non_coding( void );
 
     virtual void save( gzFile backup_file ) const;
-    
+
     int32_t get_nb_terminators( void );
-    
+
     #ifdef DEBUG
       inline void print_rna_list( void );
       inline void print_protein_list( void );
-    
+
       inline void assert_promoters( void );
       inline void assert_promoters_order( void );
     #endif
@@ -327,10 +331,10 @@ class ae_individual : public ae_object
     // pointer). Results are proportions.
     // The second is an estimate based on genome structure as defined by Carole.
     // They have been implemented on the chromosome only !
-    
-    void remove_non_coding_bases( void);
+
+    void remove_non_coding_bases(void);
     void double_non_coding_bases(void);
-    
+
 
     // =================================================================
     //                           Public Attributes
@@ -350,11 +354,6 @@ class ae_individual : public ae_object
       printf( "%s:%d: error: call to forbidden constructor.\n", __FILE__, __LINE__ );
       exit( EXIT_FAILURE );
     };
-    /*ae_individual( const ae_individual &model )
-    {
-      printf( "%s:%d: error: Call to forbidden constructor in file %s : l%d\n", __FILE__, __LINE__ );
-      exit( EXIT_FAILURE );
-    };*/
 
     // =================================================================
     //                           Protected Methods
@@ -366,14 +365,15 @@ class ae_individual : public ae_object
     //                          Protected Attributes
     // =================================================================
     ae_exp_manager* _exp_m;
-    
-    // "Age" of the strain
+
+    // Name and "Age" of the strain
+    char*   _strain_name;
     int32_t _age;
-    
+
     // Random number generator
     ae_jumping_mt* _mut_prng;
     ae_jumping_mt* _stoch_prng;
-  
+
     // Individual ID and rank of the individual in the population
     // WARNING : The ID is no longer corresponding to the rank of the individual.
     //           The reason for this change is that we now need an identifier for the individuals
@@ -381,71 +381,70 @@ class ae_individual : public ae_object
     //           The rank will now be handled in a specific new attribute. (1 for the worst indiv, POP_SIZE for the best)
     int32_t  _id;   // [0 ; POP_SIZE[
     int32_t  _rank; // [1 ; POP_SIZE]
-    
+
     // Total activation (resp. inhibition) of metabolic functions
     ae_fuzzy_set* _phenotype_activ;
     ae_fuzzy_set* _phenotype_inhib;
-    
+
     // The phenotype, roughly corresponding to the sum of activ and inhib
     ae_phenotype* _phenotype;
-    
+
     // Table containing the partial area of the "gap" (difference between the phenotype and the environment)
     // for each environmental segment.
     // Note: if the environment is not segmented, there will be a single cell in this table
     double* _dist_to_target_by_segment;
-    
+
     // This table contains the aggregated area of the gap for each type of segment (ae_env_axis_feature).
     // When the environment is not segmented, the only meaningfull value will be held in _dist_to_target[METABOLISM]
     double* _dist_to_target_by_feature;
 
     // This table contains the fitness contribution for each type of segment (ae_env_axis_feature).
-    // For metabolic segment, this is the metabolic fitness contribution; 
+    // For metabolic segment, this is the metabolic fitness contribution;
     // For secretion segment, this is the amount of compound that gets secreted into the environment;
-    // Note: total fitness is the combination of metabolic fitness and the amount of compound present in the environment, 
+    // Note: total fitness is the combination of metabolic fitness and the amount of compound present in the environment,
     //       not the amount of compound secreted by the individual.
     double* _fitness_by_feature;
 
     // THE fitness
     double _fitness;
-    
+
     // When using structured population, this is the cell the individual is in
     ae_grid_cell* _grid_cell;
-    
+
     // The chromosome and plasmids (if allowed)
     ae_list<ae_genetic_unit*>* _genetic_unit_list;
-    
+
     // Report of all the mutational events undergone during the individuals creation,
     // i.e. during the replication that gave birth to this individual
     ae_replication_report* _replic_report;
-    
+
     // Access lists to all the proteins/RNAs of the individual.
     // Please note that these proteins/RNAs are actually managed (i.e. newed and deleted) via genetic units.
     ae_list<ae_protein*>* _protein_list;
     ae_list<ae_rna*>*     _rna_list;
-    
+
     // Generic probes
     int32_t*  _int_probes;        // Table of 5 int32_t values to be used as one wishes
     double*   _double_probes;     // Table of 5 double values to be used as one wishes
-    
+
     // Mutation rates etc...
     ae_params_mut* _mut_params;
-    
+
     // ----------------------------------------------- Phenotypic stochasticity
     bool _with_stochasticity;
-    
+
     // Artificial chemistry
     double _w_max;
-      
+
     // Genome size constraints
     int32_t _min_genome_length;
     int32_t _max_genome_length;
-    
+
     // Plasmids settings
     bool _allow_plasmids;
-    int32_t _plasmid_minimal_length;
-    
-    
-    
+
+
+
     // --------------------------------------------------
     // "State" of the individual
     // --------------------------------------------------
@@ -460,11 +459,11 @@ class ae_individual : public ae_object
     bool _statistical_data_computed;
     bool _non_coding_computed;
     bool _modularity_computed;
-    
+
     bool _placed_in_population; // TODO: spatial ?
-    
-    
-    
+
+
+
     // ----------------------------------------
     // Statistical data
     // ----------------------------------------
@@ -480,9 +479,9 @@ class ae_individual : public ae_object
     int32_t _nb_non_functional_genes;           // Number of non-functional genes
     double  _overall_size_functional_genes;     // Average size of functional genes
     double  _overall_size_non_functional_genes; // Average size of non-functional genes
-    
+
     // Mutation/Rearrangement statistics are managed in the replication report
-    
+
     // Coding / non-coding
     int32_t _nb_bases_in_0_CDS;               // Number of bases that are not included in any gene
     int32_t _nb_bases_in_0_functional_CDS;    // Number of bases that are not included in any functional gene
@@ -495,9 +494,9 @@ class ae_individual : public ae_object
                                               // A base is considered neutral when neither itself NOR its corresponding base on the other
                                               // strand belongs to a coding promoter->terminator region (both included)
     int32_t _nb_neutral_regions;              // Number of neutral regions
-                                      
+
     double _modularity; // Ratio between the pairwise distance between genes whose corresponding
-                        // phenotypic triangles overlap and the average intergenic distance 
+                        // phenotypic triangles overlap and the average intergenic distance
                         // (ignoring non-functional genes)void compute_phenotype( void );
 };
 
@@ -505,6 +504,22 @@ class ae_individual : public ae_object
 // =====================================================================
 //                           Getters' definitions
 // =====================================================================
+/*!
+  TODO
+*/
+inline const char* ae_individual::get_strain_name(void) const
+{
+  return _strain_name;
+}
+
+/*!
+  TODO
+*/
+inline int32_t ae_individual::get_age(void) const
+{
+  return _age;
+}
+
 /*!
   TODO
 */
@@ -569,19 +584,19 @@ inline int16_t ae_individual::get_nb_genetic_units( void ) const
 inline int32_t ae_individual::get_amount_of_dna( void ) const
 {
   int32_t amount = 0;
-  
+
   ae_list_node<ae_genetic_unit*>* gen_unit_node = _genetic_unit_list->get_first();
   ae_genetic_unit* gen_unit = NULL;
-  
+
   while ( gen_unit_node != NULL )
   {
     gen_unit = gen_unit_node->get_obj();
-    
+
     amount += gen_unit->get_dna()->get_length();
-    
+
     gen_unit_node = gen_unit_node->get_next();
   }
-  
+
   return amount;
 }
 
@@ -599,14 +614,14 @@ inline ae_list<ae_genetic_unit*>* ae_individual::get_genetic_unit_list( void ) c
 inline ae_genetic_unit* ae_individual::get_genetic_unit( int16_t num_unit ) const
 {
   assert( num_unit < _genetic_unit_list->get_nb_elts() );
-  
+
   ae_list_node<ae_genetic_unit*>* gen_unit_node = _genetic_unit_list->get_first();
-  
+
   for ( int16_t i = 0 ; i < num_unit ; i++ )
   {
     gen_unit_node = gen_unit_node->get_next();
   }
-  
+
   return gen_unit_node->get_obj();
 }
 
@@ -616,7 +631,7 @@ inline ae_genetic_unit* ae_individual::get_genetic_unit( int16_t num_unit ) cons
 inline double ae_individual::get_dist_to_target_by_feature( ae_env_axis_feature feature ) const
 {
   assert( _distance_to_target_computed );
-  
+
   return _dist_to_target_by_feature[feature];
 }
 
@@ -626,7 +641,7 @@ inline double ae_individual::get_dist_to_target_by_feature( ae_env_axis_feature
 inline double ae_individual::get_fitness( void ) const
 {
   assert( _fitness_computed );
-  
+
   return _fitness;
 }
 
@@ -636,7 +651,7 @@ inline double ae_individual::get_fitness( void ) const
 inline double ae_individual::get_fitness_by_feature( ae_env_axis_feature feature ) const
 {
   assert( _fitness_computed );
-  
+
   return _fitness_by_feature[feature];
 }
 
@@ -710,7 +725,7 @@ ae_replication_report* ae_individual::get_replic_report( void ) const
 inline ae_list<ae_protein*>* ae_individual::get_protein_list( void ) const
 {
   assert( _protein_list );
-  
+
   return _protein_list;
 }
 
@@ -768,7 +783,7 @@ inline int32_t ae_individual::get_nb_non_coding_RNAs( void )
 /*!
   TODO
 */
-inline double ae_individual::get_overall_size_coding_RNAs( void ) 
+inline double ae_individual::get_overall_size_coding_RNAs( void )
 {
   if ( ! _statistical_data_computed ) compute_statistical_data();
   return _overall_size_coding_RNAs;
@@ -777,7 +792,7 @@ inline double ae_individual::get_overall_size_coding_RNAs( void )
 /*!
   TODO
 */
-inline double ae_individual::get_av_size_coding_RNAs( void ) 
+inline double ae_individual::get_av_size_coding_RNAs( void )
 {
   if ( ! _statistical_data_computed ) compute_statistical_data();
   if ( _nb_coding_RNAs != 0 )
@@ -790,7 +805,7 @@ inline double ae_individual::get_av_size_coding_RNAs( void )
 /*!
   TODO
 */
-inline double ae_individual::get_overall_size_non_coding_RNAs( void ) 
+inline double ae_individual::get_overall_size_non_coding_RNAs( void )
 {
   if ( ! _statistical_data_computed ) compute_statistical_data();
   return _overall_size_non_coding_RNAs;
@@ -830,7 +845,7 @@ inline int32_t ae_individual::get_nb_genes_inhib( void )
 /*!
   TODO
 */
-inline int32_t ae_individual::get_nb_functional_genes( void ) 
+inline int32_t ae_individual::get_nb_functional_genes( void )
 {
   if ( ! _statistical_data_computed ) compute_statistical_data();
   return _nb_functional_genes;
@@ -839,7 +854,7 @@ inline int32_t ae_individual::get_nb_functional_genes( void )
 /*!
   TODO
 */
-inline int32_t ae_individual::get_nb_non_functional_genes( void ) 
+inline int32_t ae_individual::get_nb_non_functional_genes( void )
 {
   if ( ! _statistical_data_computed ) compute_statistical_data();
   return _nb_non_functional_genes;
@@ -848,7 +863,7 @@ inline int32_t ae_individual::get_nb_non_functional_genes( void )
 /*!
   TODO
 */
-inline double ae_individual::get_overall_size_functional_genes( void ) 
+inline double ae_individual::get_overall_size_functional_genes( void )
 {
   if ( ! _statistical_data_computed ) compute_statistical_data();
   return _overall_size_functional_genes;
@@ -857,7 +872,7 @@ inline double ae_individual::get_overall_size_functional_genes( void )
 /*!
   TODO
 */
-inline double ae_individual::get_av_size_functional_genes( void ) 
+inline double ae_individual::get_av_size_functional_genes( void )
 {
   if ( ! _statistical_data_computed ) compute_statistical_data();
   if ( _nb_functional_genes != 0 )
@@ -870,7 +885,7 @@ inline double ae_individual::get_av_size_functional_genes( void )
 /*!
   TODO
 */
-inline double ae_individual::get_overall_size_non_functional_genes( void ) 
+inline double ae_individual::get_overall_size_non_functional_genes( void )
 {
   if ( ! _statistical_data_computed ) compute_statistical_data();
   return _overall_size_non_functional_genes;
@@ -879,7 +894,7 @@ inline double ae_individual::get_overall_size_non_functional_genes( void )
 /*!
   TODO
 */
-inline double ae_individual::get_av_size_non_functional_genes( void ) 
+inline double ae_individual::get_av_size_non_functional_genes( void )
 {
   if ( ! _statistical_data_computed ) compute_statistical_data();
   if ( _nb_non_functional_genes != 0 )
@@ -974,7 +989,7 @@ inline int32_t ae_individual::get_nb_neutral_regions( void )
 */
 inline double ae_individual::get_modularity( void )
 {
-  printf("\n  WARNING : modularity measure not yet implemented.\n"); 
+  printf("\n  WARNING : modularity measure not yet implemented.\n");
   //~ if ( _modularity < 0 ) compute_modularity();
   //~ return _modularity;
   return 0;
@@ -987,7 +1002,7 @@ inline double ae_individual::get_w_max( void ) const
 {
   return _w_max;
 }
-    
+
 // ------------------------------------------------------------- Mutation rates
 /*!
   TODO
@@ -1099,34 +1114,34 @@ inline double ae_individual::get_inversion_proportion( void ) const
 inline bool ae_individual::get_with_4pts_trans( void ) const
 {
   return _mut_params->get_with_4pts_trans();
-}  
+}
 
 inline bool ae_individual::get_with_HT( void ) const
 {
   return _mut_params->get_with_HT();
-}  
+}
 
 inline bool ae_individual::get_repl_HT_with_close_points( void ) const
 {
   return _mut_params->get_repl_HT_with_close_points();
-}  
+}
 
 inline double ae_individual::get_HT_ins_rate( void ) const
 {
   return _mut_params->get_HT_ins_rate();
-}  
+}
 
 inline double ae_individual::get_HT_repl_rate( void ) const
 {
   return _mut_params->get_HT_repl_rate();
-}  
+}
 
 inline double ae_individual::get_repl_HT_detach_rate( void ) const
 {
   return _mut_params->get_repl_HT_detach_rate();
-}  
+}
+
 
-    
 // ---------------------------------------------------------------- Alignements
 inline bool ae_individual::get_with_alignments(void) const
 {
@@ -1212,17 +1227,9 @@ inline bool ae_individual::get_allow_plasmids( void ) const
   return _allow_plasmids;
 }
 
-/*!
-  TODO
-*/
-inline int32_t ae_individual::get_plasmid_minimal_length( void ) const
-{
-  return _plasmid_minimal_length;
-}
-
 /*!
   \brief Return the _int_probes
-  
+
   \return _int_probes
 */
 inline int32_t* ae_individual::get_int_probes ( void ) const
@@ -1232,29 +1239,34 @@ inline int32_t* ae_individual::get_int_probes ( void ) const
 
 /*!
   \brief Return the _double_probes
-  
+
   \return _double_probes
 */
 inline double* ae_individual::get_double_probes ( void) const
 {
   return _double_probes;
 }
-    
-    
+
+
 // =====================================================================
 //                           Setters' definitions
 // =====================================================================
-inline void ae_individual::set_exp_m( ae_exp_manager* exp_m )
+inline void ae_individual::set_strain_name(char* name)
 {
-  _exp_m = exp_m;
+  assert(name && strlen(name) < INT8_MAX); // Conservative, could be <=
+  int8_t name_len = strlen(name);
+  delete [] _strain_name;
+  _strain_name = new char[name_len+1];
+  memcpy(_strain_name, name, name_len+1);
 }
+
 /*!
   TODO
 */
 inline void ae_individual::set_id( int32_t id )
 {
   _id = id;
-  
+
   if ( _replic_report != NULL )
   {
     _replic_report->set_id( id );
@@ -1267,7 +1279,7 @@ inline void ae_individual::set_id( int32_t id )
 inline void ae_individual::set_rank( int32_t rank )
 {
   _rank = rank;
-  
+
   if ( _replic_report != NULL )
   {
     _replic_report->set_rank( rank );
@@ -1443,7 +1455,7 @@ inline void ae_individual::set_mut_prng( ae_jumping_mt* prng )
 
 /*!
   \brief Change the _int_probes
-  
+
   \param int_probes 5 int32_t* that constitute a probe
 */
 inline void ae_individual::set_int_probes ( int32_t* int_probes)
@@ -1453,7 +1465,7 @@ inline void ae_individual::set_int_probes ( int32_t* int_probes)
 
 /*!
   \brief Change the _double_probes
-  
+
   \param double_probes 5 double* that constitute a probe
 */
 inline void ae_individual::set_double_probes ( double* double_probes)
@@ -1487,11 +1499,11 @@ inline void ae_individual::renew_fitness_by_feature( void )
 void ae_individual::do_transcription_translation_folding( void )
 {
   if ( _transcribed == true && _translated == true && _folded == true ) return;
-  
+
   do_transcription();
   do_translation();
   do_folding();
-  
+
   make_protein_list();
 }
 
@@ -1500,64 +1512,64 @@ void ae_individual::do_transcription_translation_folding( void )
   {
     ae_list_node<ae_rna*>* rna_node  = _rna_list->get_first();
     ae_rna*       rna       = NULL;
-    
+
     while ( rna_node != NULL )
     {
       rna = rna_node->get_obj();
-      
+
       printf( "RNA at pos : %"PRId32"      length : %"PRId32" bp\n", rna->get_promoter_pos(), rna->get_transcript_length() );
       printf( "  strand : %s    basal level : %f\n", (rna->get_strand() == LEADING)?"LEADING":"LAGGING", rna->get_basal_level() );
-      
+
       rna_node = rna_node->get_next();
     }
   }
-  
+
   inline void ae_individual::print_protein_list( void )
   {
     ae_list_node<ae_protein*>* prot_node = _protein_list->get_first();
     ae_protein*   prot = NULL;
-    
+
     while ( prot_node != NULL )
     {
       prot = prot_node->get_obj();
-      
+
       char* prot_sequence = prot->get_AA_sequence();
-      printf( "prot at pos : %"PRId32"      length : %"PRId32" AAs\n", prot->get_first_translated_pos(), prot->get_length() );
+      printf( "prot %p at pos : %"PRId32"      length : %"PRId32" AAs\n", prot, prot->get_first_translated_pos(), prot->get_length() );
       printf( "  strand : %s    concentration : %f\n", (prot->get_strand() == LEADING)?"LEADING":"LAGGING", prot->get_concentration() );
       printf( "  sequence : %s\n", prot_sequence );
       printf( "  mean : %f      width : %f      height : %f\n", prot->get_mean(), prot->get_width(), prot->get_height() );
       delete prot_sequence;
-      
+
       prot_node = prot_node->get_next();
     }
   }
-  
+
   inline void ae_individual::assert_promoters( void )
   {
     // Perform assertion for each genetic unit
     ae_list_node<ae_genetic_unit*>*     gen_unit_node = _genetic_unit_list->get_first();
     ae_genetic_unit*  gen_unit      = NULL;
-    
+
     while ( gen_unit_node != NULL )
     {
       gen_unit = gen_unit_node->get_obj();
-      
+
       gen_unit->assert_promoters();
 
       gen_unit_node = gen_unit_node->get_next();
     }
   }
-  
+
   inline void ae_individual::assert_promoters_order( void )
   {
     // Perform assertion for each genetic unit
     ae_list_node<ae_genetic_unit*>*     gen_unit_node = _genetic_unit_list->get_first();
     ae_genetic_unit*  gen_unit      = NULL;
-    
+
     while ( gen_unit_node != NULL )
     {
       gen_unit = gen_unit_node->get_obj();
-      
+
       gen_unit->assert_promoters_order();
 
       gen_unit_node = gen_unit_node->get_next();
