@@ -352,6 +352,7 @@ ae_mutation::~ae_mutation( void )
   {
     case SWITCH :
       delete _pos;
+      delete _length;
       break;
     case S_INS :
       delete _pos;
@@ -523,8 +524,8 @@ void ae_mutation::report_ins_HT(int32_t alignment_1_donor_pos_1, int32_t alignme
   _pos = new int32_t[4];
   _pos[0] = alignment_1_donor_pos_1; //donor
   _pos[1] = alignment_1_donor_pos_2; //donor
-  _pos[2] = alignment_2_ind_pos; //exogenote
-  _pos[3] = alignment_2_donor_pos; //receiver
+  _pos[2] = alignment_2_ind_pos; //exogenote   // NOTE: wrong comment? I think it's receiver
+  _pos[3] = alignment_2_donor_pos; //receiver  // NOTE: wrong comment? I think it's exogenote
   
   _length = new int32_t( length );
   _seq = new char[length + 1];
@@ -700,19 +701,19 @@ void ae_mutation::get_generic_description_string( char * str )
     case INSERT :
     {
       sprintf( str, "%"PRId8" %"PRId32" %"PRId32" %"PRId32" %"PRId32" %"PRId8" %"PRId16" %"PRId16" %"PRId32" %"PRId32"", \
-              (int8_t) _mut_type, _pos[0], (int32_t)-1, (int32_t)-1, (int32_t)-1, (int8_t) -1, (int16_t) -1, (int32_t) -1, _length[0], (int32_t) -1 );
+              (int8_t) _mut_type, _pos[0], (int32_t)-1, (int32_t)-1, (int32_t)-1, (int8_t) -1, (int16_t) -1, (int16_t) -1, _length[0], (int32_t) -1 );
       break;
     }
     case INS_HT :
     {
       sprintf( str, "%"PRId8" %"PRId32" %"PRId32" %"PRId32" %"PRId32" %"PRId8" %"PRId16" %"PRId16" %"PRId32" %"PRId32"", \
-              (int8_t) _mut_type, _pos[0], _pos[1], _pos[2], _pos[3], _sense, _align_score[0], _align_score[1], _length[0], (int32_t) -1 );
+              (int8_t) _mut_type, _pos[0], _pos[1], _pos[2], _pos[3], (int8_t) _sense, _align_score[0], _align_score[1], _length[0], (int32_t) -1 );
       break;
     }
     case REPL_HT :
     {
       sprintf( str, "%"PRId8" %"PRId32" %"PRId32" %"PRId32" %"PRId32" %"PRId8" %"PRId16" %"PRId16" %"PRId32" %"PRId32" ", \
-              (int8_t) _mut_type, _pos[0], _pos[1], _pos[2], _pos[3], _sense, _align_score[0], _align_score[1], _length[1], _length[0] );
+              (int8_t) _mut_type, _pos[0], _pos[1], _pos[2], _pos[3], (int8_t) _sense, _align_score[0], _align_score[1], _length[1], _length[0] );
       break;
     }
     default :
@@ -730,7 +731,7 @@ void ae_mutation::get_generic_description_string( char * str )
 /* DEPRECATED, use get_length instead */
 int32_t ae_mutation::segment_length( int32_t gen_unit_len )
 {
-  int32_t res;
+  int32_t res = 0;
   switch ( _mut_type )
   {
     case SWITCH :
