@@ -122,7 +122,7 @@ void ae_output_manager::write_setup_file( FILE* setup_file ) const
   fprintf( setup_file, "BIG_BACKUP_STEP %"PRId32"\n", _big_backup_step );
   
   // Stats
-  fprintf( setup_file, "COMPUTE_PHENOTYPIC_CONTRIBUTION_BY_GU %"PRId8"\n", _compute_phen_contrib_by_GU );
+  fprintf( setup_file, "COMPUTE_PHENOTYPIC_CONTRIBUTION_BY_GU %"PRId8"\n", (int8_t) _compute_phen_contrib_by_GU );
   
   // Tree
   fprintf( setup_file, "RECORD_TREE %s\n", _record_tree ? "true" : "false" );
@@ -161,7 +161,7 @@ void ae_output_manager::load( gzFile setup_file, bool verbose, bool to_be_run  )
   
   // Stats
   int32_t num_gener = _exp_m->get_num_gener();
-  if( to_be_run)
+  if( to_be_run )
   {
     if ( num_gener > 0 )
     {
@@ -186,8 +186,8 @@ void ae_output_manager::load( gzFile setup_file, bool verbose, bool to_be_run  )
     gzread( setup_file, &tmp_tree_mode, sizeof(tmp_tree_mode) );
     if ( (ae_tree_mode)tmp_tree_mode != LIGHT && (ae_tree_mode)tmp_tree_mode != NORMAL)
     {
-        printf( "%s:%d: error: invalid tree mode\n", __FILE__, __LINE__ );
-        exit( EXIT_FAILURE );
+      printf( "%s:%d: error: invalid tree mode\n", __FILE__, __LINE__ );
+      exit( EXIT_FAILURE );
     }
     
     _tree = new ae_tree( _exp_m, (ae_tree_mode) tmp_tree_mode, tmp_tree_step );
@@ -231,7 +231,7 @@ void ae_output_manager::load( FILE* setup_file, bool verbose, bool to_be_run  )
       _stats = new ae_stats( _exp_m );
     }
   }
-  fscanf( setup_file, "COMPUTE_PHENOTYPIC_CONTRIBUTION_BY_GU %"PRId8"\n", &_compute_phen_contrib_by_GU );
+  fscanf( setup_file, "COMPUTE_PHENOTYPIC_CONTRIBUTION_BY_GU %"PRId8"\n", (int8_t*) &_compute_phen_contrib_by_GU );
   
   char tmp[10];
   
@@ -336,7 +336,7 @@ void ae_output_manager::write_tree( void ) const
   status = mkdir( TREE_DIR, 0755 );
   if ( (status == -1) && (errno != EEXIST) )
   {
-    err( EXIT_FAILURE, TREE_DIR, errno );
+    err( EXIT_FAILURE, "Impossible to create the directory %s", TREE_DIR );
   }
   
   char tree_file_name[50];
