@@ -3,25 +3,25 @@
 //          Aevol - An in silico experimental evolution platform
 //
 // ****************************************************************************
-// 
+//
 // Copyright: See the AUTHORS file provided with the package or <www.aevol.fr>
 // Web: http://www.aevol.fr/
 // E-mail: See <http://www.aevol.fr/contact/>
 // Original Authors : Guillaume Beslon, Carole Knibbe, David Parsons
-// 
+//
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
 // the Free Software Foundation, either version 2 of the License, or
 // (at your option) any later version.
-// 
+//
 // This program is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
-// 
+//
 // You should have received a copy of the GNU General Public License
 // along with this program.  If not, see <http://www.gnu.org/licenses/>.
-// 
+//
 //*****************************************************************************
 
 
@@ -76,13 +76,13 @@
 ae_population::ae_population( ae_exp_manager* exp_m )
 {
   _exp_m = exp_m;
-  
+
   #ifndef DISTRIBUTED_PRNG
     _mut_prng       = NULL;
     _stoch_prng     = NULL;
     _stoch_prng_bak = NULL;
   #endif
-  
+
   // Individuals
   _nb_indivs  = 0;
   _indivs     = new ae_list<ae_individual*>();
@@ -100,7 +100,7 @@ ae_population::~ae_population( void )
     delete _stoch_prng;
     delete _stoch_prng_bak;
   #endif
-  
+
   _indivs->erase( true );
   delete _indivs;
 }
@@ -110,41 +110,55 @@ ae_population::~ae_population( void )
 // =================================================================
 void ae_population::set_nb_indivs(int32_t nb_indivs)
 {
-	int32_t index_to_duplicate;
-	ae_individual* indiv = NULL;
-	if(nb_indivs > _nb_indivs)
-	{
-		int32_t initial_pop_size = _nb_indivs;
-		for(int32_t i = initial_pop_size; i < nb_indivs; i++)
-		{
-			index_to_duplicate = _exp_m->get_sel()->get_prng()->random( initial_pop_size ) + 1;
-			indiv = new ae_individual(*get_indiv_by_id(index_to_duplicate));
-			indiv->set_id(i);
-			add_indiv(indiv);
-		}
-	}
-	else if(nb_indivs < _nb_indivs)
-	{
-		ae_list<ae_individual*>* new_population = new ae_list<ae_individual*>();
-		for(int32_t i = 0; i < nb_indivs; i++)
-		{
-			index_to_duplicate = _exp_m->get_sel()->get_prng()->random( _nb_indivs ) + 1;
-			indiv = new ae_individual(*get_indiv_by_id(index_to_duplicate));
-			indiv->set_id(i);
-			new_population->add(indiv);
-		}
-		replace_population(new_population);
-	}
-	sort_individuals();
+  int32_t index_to_duplicate;
+  ae_individual* indiv = NULL;
+  if(nb_indivs > _nb_indivs)
+  {
+    int32_t initial_pop_size = _nb_indivs;
+    for(int32_t i = initial_pop_size; i < nb_indivs; i++)
+    {
+      index_to_duplicate = _exp_m->get_sel()->get_prng()->random( initial_pop_size );
+      indiv = new ae_individual(*get_indiv_by_id(index_to_duplicate), true);
+      indiv->set_id(i);
+      add_indiv(indiv);
+    }
+  }
+  else if(nb_indivs < _nb_indivs)
+  {
+    ae_list<ae_individual*>* new_population = new ae_list<ae_individual*>();
+    for(int32_t i = 0; i < nb_indivs; i++)
+    {
+      index_to_duplicate = _exp_m->get_sel()->get_prng()->random( _nb_indivs );
+      indiv = new ae_individual(*get_indiv_by_id(index_to_duplicate), true);
+      indiv->set_id(i);
+      new_population->add(indiv);
+    }
+    update_population(new_population);
+  }
+  sort_individuals();
 }
 
-void ae_population::replace_population( ae_list<ae_individual*>* new_indivs )
+void ae_population::replace_population(ae_list<ae_individual*>* new_indivs)
 {
-  _indivs->erase( true );
-  delete _indivs;
-  
-  _indivs = new_indivs;
-  _nb_indivs = _indivs->get_nb_elts();
+  // First replace the former indivs by the new ones
+  update_population(new_indivs);
+
+  // Then reconciliate any possible inconsistency...
+
+  // Update pointer to exp_manager in each individual
+  // Replace indivs id by a new one
+  ae_list_node<ae_individual*>* indiv_node = _indivs->get_first();
+  ae_individual* indiv = NULL;
+  int32_t  id = 0;
+  while ( indiv_node != NULL )
+  {
+    indiv = indiv_node->get_obj();
+
+    indiv->set_exp_m(_exp_m);
+    indiv->set_id(id++);
+
+    indiv_node = indiv_node->get_next();
+  }
 }
 
 void ae_population::save( gzFile backup_file ) const
@@ -160,7 +174,7 @@ void ae_population::save( gzFile backup_file ) const
     }
   #endif
   gzwrite( backup_file, &_nb_indivs, sizeof(_nb_indivs) );
-  
+
   // Write individuals
   ae_list_node<ae_individual*>*   indiv_node = _indivs->get_first();
   ae_individual*  indiv;
@@ -189,14 +203,15 @@ void ae_population::load( gzFile backup_file, bool verbose )
   // ----------------------------------------------------- Retreive individuals
   if ( verbose ) printf( "  Loading individuals " );
   ae_individual* indiv = NULL;
+  int32_t nb_ind_div_10 = _nb_indivs / 10;
   for ( int32_t i = 0 ; i < _nb_indivs ; i++ )
   {
-    if ( verbose && i && i % 100 == 0 )
+    if ( verbose && i && i % nb_ind_div_10 == 0 )
     {
       putchar( '*' );
       fflush( stdout );
     }
-    
+
     #ifdef __NO_X
       #ifndef __REGUL
         indiv = new ae_individual( _exp_m, backup_file );
@@ -210,12 +225,23 @@ void ae_population::load( gzFile backup_file, bool verbose )
         indiv = new ae_individual_R_X11( _exp_m, backup_file );
       #endif
     #endif
-    
+
     _indivs->add( indiv );
   }
-  if ( verbose ) putchar( '\n' );
 }
-  
+
+void ae_population::load(const char* backup_file_name, bool verbose)
+{
+  gzFile backup_file = gzopen(backup_file_name, "r");
+  if ( backup_file == Z_NULL )
+  {
+    printf( "%s:%d: error: could not open pop file %s\n",
+            __FILE__, __LINE__, backup_file_name );
+    exit( EXIT_FAILURE );
+  }
+  this->load(backup_file, verbose);
+}
+
 #ifndef DISTRIBUTED_PRNG
   void ae_population::backup_stoch_prng( void )
   {
@@ -251,7 +277,7 @@ void ae_population::sort_individuals( void )
     while ( !is_sorted )
     {
       fit_comp = comp->get_obj()->get_fitness();
-      
+
       if ( fitness_to_sort >= fit_comp ) // The right place of the item is after comp
       {
         if ( item_to_sort->get_prev() == comp )
@@ -271,7 +297,7 @@ void ae_population::sort_individuals( void )
         // move on to compare with the next item
         comp = comp->get_prev();
       }
-      
+
       if ( comp == NULL )
       {
         // item_to_sort has to be inserted at the beginning of the list
@@ -281,18 +307,18 @@ void ae_population::sort_individuals( void )
       }
     }
   }
-  
+
   // Update the rank of the individuals
   ae_list_node<ae_individual*>* indiv_node  = _indivs->get_first();
   ae_individual* indiv       = NULL;
-  
+
   for ( int32_t rank = 1 ; rank <= _nb_indivs ; rank++ )
   {
     indiv = indiv_node->get_obj();
     indiv->set_rank( rank );
     indiv_node = indiv_node->get_next();
   }
-  
+
 }
 
 // Find the best individual and put it at the end of the list: this is quicker than sorting the whole list in case we only need the best individual, for example when we have spatial structure.
@@ -300,7 +326,7 @@ void ae_population::update_best( void )
 {
   ae_list_node<ae_individual*>* current_best  = _indivs->get_first();
   ae_list_node<ae_individual*>* candidate     = _indivs->get_first();
-  
+
   while ( candidate != NULL )
   {
     if (  candidate->get_obj()->get_fitness() >= current_best->get_obj()->get_fitness() )
@@ -309,33 +335,46 @@ void ae_population::update_best( void )
     }
     candidate = candidate->get_next();
   }
-  
+
   _indivs->remove( current_best, false, false );
   _indivs->add( current_best );
 
   current_best->get_obj()->set_rank( _nb_indivs );
 }
 
+// The new pop must be consistent and belong to the same experiment as the one
+// it replaces, otherwise use replace_population
+void ae_population::update_population(ae_list<ae_individual*>* new_indivs)
+{
+  // Delete the former indivs
+  _indivs->erase( true );
+  delete _indivs;
+
+  // Replace with new indivs
+  _indivs = new_indivs;
+  _nb_indivs = _indivs->get_nb_elts();
+}
+
 ae_individual* ae_population::create_clone( ae_individual* dolly, int32_t id )
 {
   ae_individual* indiv;
-  
+
   #ifdef __NO_X
     #ifndef __REGUL
-      indiv = new ae_individual( *dolly );
+      indiv = new ae_individual( *dolly, false );
     #else
-      indiv = new ae_individual_R( *(dynamic_cast<ae_individual_R*>(dolly)) );
+      indiv = new ae_individual_R( *(dynamic_cast<ae_individual_R*>(dolly)) , false);
     #endif
   #elif defined __X11
     #ifndef __REGUL
-      indiv = new ae_individual_X11( *(dynamic_cast<ae_individual_X11*>(dolly)) );
+      indiv = new ae_individual_X11( *(dynamic_cast<ae_individual_X11*>(dolly)), false );
     #else
-      indiv = new ae_individual_R_X11( *(dynamic_cast<ae_individual_R_X11*>(dolly)) );
+      indiv = new ae_individual_R_X11( *(dynamic_cast<ae_individual_R_X11*>(dolly)), false );
     #endif
   #endif
-  
+
   indiv->set_id( id );
-  
+
   return indiv;
 }
 
@@ -353,14 +392,14 @@ ae_individual* ae_population::get_indiv_by_id( int32_t id ) const
   while ( indiv_node != NULL )
   {
     indiv = indiv_node->get_obj();
-    
+
     if ( indiv->get_id() == id )
     {
       return indiv;
     }
-    
+
     indiv_node = indiv_node->get_next();
   }
-  
+
   return NULL;
 }
