@@ -68,8 +68,15 @@ class ae_stats : public ae_object
     // =================================================================
     //                             Constructors
     // =================================================================
-    ae_stats( ae_exp_manager* exp_m, const char * prefix = "stat", bool best_indiv_only = false );
-    ae_stats( ae_exp_manager* exp_m, int32_t num_gener, const char * prefix = "stat", bool best_indiv_only = false, bool addition_old_stats = true, bool delete_old_stats = true );
+    ae_stats( ae_exp_manager* exp_m,
+              bool best_indiv_only = false,
+              const char * prefix = "stat" );
+    ae_stats( ae_exp_manager* exp_m,
+              int32_t num_gener,
+              bool best_indiv_only = false,
+              const char * prefix = "stat",
+              bool addition_old_stats = true,
+              bool delete_old_stats = true );
 
  
     // =================================================================
