@@ -88,7 +88,8 @@ ae_tree::ae_tree( ae_exp_manager* exp_m, ae_tree_mode tree_mode, int32_t tree_st
     }
     case LIGHT :
     {
-      #warning LIGHT tree mode: constructor and destructor commented
+      // TO DO
+
       //~ // Creates the big arrays to store number of individuals and child->parent relations
       //~ _nb_indivs  = new int32_t[ae_common::nb_generations];
       //~ _parent     = new int32_t*[ae_common::nb_generations];
@@ -284,26 +285,27 @@ void ae_tree::fill_tree_with_cur_gener( void )
     }
     case LIGHT :
     {
+      // TO DO
 
       // TO CHECK !!
       // not sure that gener_i should be used in this block...
 
-      int32_t gener_i     = ae_utils::mod( _exp_m->get_num_gener() - 1, _tree_step );
-      _nb_indivs[gener_i] = _exp_m->get_nb_indivs();
-      _parent[gener_i] = new int32_t [_nb_indivs[gener_i]];
+      // int32_t gener_i     = ae_utils::mod( _exp_m->get_num_gener() - 1, _tree_step );
+      // _nb_indivs[gener_i] = _exp_m->get_nb_indivs();
+      // _parent[gener_i] = new int32_t [_nb_indivs[gener_i]];
       
-      ae_list_node<ae_individual*>* indiv_node = _exp_m->get_indivs()->get_first();
-      ae_individual*  indiv       = NULL;
-      int32_t         num_indiv   = 0;
+      // ae_list_node<ae_individual*>* indiv_node = _exp_m->get_indivs()->get_first();
+      // ae_individual*  indiv       = NULL;
+      // int32_t         num_indiv   = 0;
       
-      while( indiv_node != NULL )
-      {
-        indiv = indiv_node->get_obj();
+      // while( indiv_node != NULL )
+      // {
+      //   indiv = indiv_node->get_obj();
         
-        _parent[gener_i][num_indiv++] = indiv->get_replic_report()->get_parent_id();
+      //   _parent[gener_i][num_indiv++] = indiv->get_replic_report()->get_parent_id();
         
-        indiv_node = indiv_node->get_next();
-      }
+      //   indiv_node = indiv_node->get_next();
+      // }
       
       break;
     }
@@ -323,6 +325,7 @@ void ae_tree::write_to_tree_file( gzFile tree_file )
       {
         for ( int32_t indiv_i = 0 ; indiv_i < _nb_indivs[gener_i] ; indiv_i++ )
         {
+          assert(_replics[gener_i][indiv_i] != NULL);
           _replics[gener_i][indiv_i]->write_to_tree_file( tree_file );
         }
       }
@@ -359,7 +362,7 @@ void ae_tree::set_replic_report( int32_t id, ae_replication_report* replic_repor
 {
   assert( _tree_mode == NORMAL );
   
-  int32_t gener_i = _exp_m->get_num_gener() % _tree_step;
+  int32_t gener_i = ae_utils::mod(_exp_m->get_num_gener() - 1, _tree_step); // CK: BUGFIX. Previous expression was: _exp_m->get_num_gener() % _tree_step;
   
   if ( _replics[gener_i] == NULL )
   {
@@ -375,6 +378,38 @@ void ae_tree::set_replic_report( int32_t id, ae_replication_report* replic_repor
 
 
 
+// CK: Added for aevol_modify
+void ae_tree::set_replic_report( int32_t generation, int32_t id, ae_replication_report* replic_report )
+{
+  assert( _tree_mode == NORMAL );
+
+  int32_t g = ae_utils::mod(generation - 1, _tree_step);
+
+  if ( _replics[g] == NULL )
+  {
+    _replics[g] = new ae_replication_report* [_exp_m->get_nb_indivs()];
+    memset( _replics[g], 0, _exp_m->get_nb_indivs() * sizeof( *_replics ) );  
+  }
+
+  if ( _replics[g][id] != NULL )
+    {
+      printf("Erased previous replication report for indiv %d of generation %d (%d)\n", id, generation, g);
+      delete _replics[g][id];
+    }
+  _replics[g][id] = new ae_replication_report(*replic_report);
+  _replics[g][id]->set_id(id);
+
+  // debug
+  printf("Added replication report for indiv %d of generation %d (%d) :\n", id, generation, g);
+  printf("  ID            %d \n", _replics[g][id]->get_id() );
+  printf("  Rank          %d \n", _replics[g][id]->get_rank() );
+  printf("  Genome size   %d \n", _replics[g][id]->get_genome_size() );
+  printf("  P. ID         %d \n", _replics[g][id]->get_parent_id() );
+  printf("  P. met.err.   %f \n", _replics[g][id]->get_parent_metabolic_error() );
+  printf("  P. size       %d \n", _replics[g][id]->get_parent_genome_size() );
+}
+
+
 
 
 // =================================================================
